#ifndef _EDW_GFX_H_
#define	_EDW_GFX_H_


#include <Arduino.h>

//----- Atrybuty -----//
#define  TFT_MODE_PROP       1   // Czcionka proporcjonalna
#define TFT_MODE_TXT_BACK   2   // czcionka z tlem


//----- Komendy -----//
#define DISP_CFG_8      0     // Kolory 8-bit
#define DISP_CFG_4      1     // 4-bit
#define DISP_CFG_16     2     // 16-bit
//
#define CMDd_RW        0x80    // Maska bitowa bitu zapis/odczyt - 0 zapis
#define CMDd_CD        0x40    // Maska bitowa komenda/dane - nastepna paczka bedzie komenda (1) czy danymi (0)
#define CMDd_8_16      0x20    // Maska bitowa operacji 8/16-bit: 0 - 8-bit
//
#define CMDd_NOP          0   // 
#define CMDd_OFS          2   // Ofset wspolrzednych x, y umozliwia "chowanie" obiektow na gorze i z lewej strony ekranu
#define CMDd_SWRESET      1
#define CMDd_CFG         3
#define CMDd_RESOLUTION  4     // Rozdzielcosc wyswietlacza: w(szerokosc), h(wysokosc)
#define CMDd_COLOR_TAB   5     // Tablica kolorow RGB. 16 dla 4-bit, 256 dla 8-bit
#define CMDd_CLS         6     // Czysc LCD: kolor
#define CMDd_DISPLAY     7     // Wyswietl dane z bufora na LCD
#define CMDd_WINDOW     8     // Zdefiniowanie okna: x, y, w(szerokosc), h(wysokosc)
#define CMDd_PIXEL       9     // Piksel: x, y, kolor. Ewentualne kolejne dane piksele zgodnie z konfiguracja i w zdefioniowanym oknie
#define CMDd_CURSOR      10     // j.w. bez ustawiania koloru
#define CMDd_COLOR        11    // Kolor piora, tla
#define CMDd_OUT         12     // Ustawienie wyjsc
#define CMDd_SPRITE       13     // Jak WINDOW ale kolor 0 jest przezrocysty
#define CMDd_PWM         14     // Ustawienie wyjscia PWM
#define CMDd_STRING       15     // Duszki - jak CMDd_WINDOW tyle, ze kolor 0 przezroczysty
#define CMDd_FILL       16     // Wypelnianie obszaru kolorem
#define CMDd_LINE_H       17
#define CMDd_LINE_V       18
#define CMDd_LINE       19


#define GFX_TURBO_ID_SIZE   38


/*
// makro zwracające 24-bitowy wskaźnik do zmiennej ‘var’
#define get_memx_address(var)               \
({                                          \
    __int24 tmp;                            \
                                            \
    __asm__ __volatile__(                   \
                                            \
    "ldi        %A0, lo8(%1)"       "\n\t"      \
    "ldi        %B0, hi8(%1)"       "\n\t"      \
    "ldi        %C0, hh8(%1)"       "\n\t"      \
    :                                       \
    "=d" (tmp)                              \
    :                                       \
    "p"  (&(var))                           \
    );                                      \
    (__memx typeof(var)*)tmp;               \
})
*/


class EdWgfx  {
  public:
    EdWgfx( uint8_t csPin, uint8_t busyPin );

    void EdWgfx::readId( uint8_t *buf );
    void display();
    void begin();
    void ofs( uint8_t ofsX, uint8_t ofsY );
    void nop();
    void fillScreen( uint8_t kolor );
    void setAddrWindow(uint8_t x0, uint8_t y0, uint8_t x1, uint8_t y1);
    void drawSprite(uint8_t x0, uint8_t y0, uint8_t x1, uint8_t y1, uint8_t * dat, uint16_t len);
    void drawSprite_P(uint8_t x0, uint8_t y0, uint8_t x1, uint8_t y1, uint8_t * dat, uint16_t len);
    void drawSprite_PF(uint8_t x0, uint8_t y0, uint8_t x1, uint8_t y1, uint8_t * dat, uint16_t len);
//    void drawSprite_A(uint8_t x0, uint8_t y0, uint8_t x1, uint8_t y1, __memx * dat, uint16_t len);
    void setCursor(uint8_t x, uint8_t y);
    void drawPixel(uint8_t x, uint8_t y, uint8_t color);
    void drawFastVLine(uint8_t x, uint8_t y, uint8_t h, uint8_t color);
    void drawFastHLine(uint8_t x, uint8_t y, uint8_t w, uint8_t color);
    void drawLine(int16_t x0, int16_t y0, int16_t x1, int16_t y1, uint8_t color);
    void drawRect(uint8_t x, uint8_t y, uint8_t w, uint8_t h, uint8_t color);
    void fillRect(uint8_t x, uint8_t y, uint8_t w, uint8_t h, uint8_t color);
    void setTextColor(uint8_t color);
    void setTextBack(uint8_t color);
    void setTextSize(uint8_t rozmiar);
    void setTextAtrybut(uint8_t atrybuty);
    void printString(char *txt);
    void setContinue(uint8_t cont);
    uint8_t width();
    uint8_t height();
    void setRotation(uint8_t r);
    void setOUT(uint8_t r);
    void setPWM(uint8_t r);
    uint8_t readOUT();
    uint8_t readIN();
    uint8_t readPWM();
    uint16_t readAdrDMA();


  private:
    void Send4spi(uint8_t s1, uint8_t s2, uint8_t s3, uint8_t s4, uint8_t cs);
    void SendReciv4spi( uint8_t s1, uint8_t s2, uint8_t s3, uint8_t s4, uint8_t *r1, uint8_t *r2, uint8_t *r3, uint8_t *r4, uint8_t cs );
    uint8_t csPin_, busyPin_;
    uint16_t CursX, CursY;    
    uint16_t rozdzielczoscX, rozdzielczoscY;    
    uint8_t ColorPen, ColorBack, txtSize, txtAtrybuty;    
    uint8_t cont;
};


#endif
